@@ -408,13 +408,15 @@ void tiffWriter(gdImagePtr image, gdIOCtx *out, int bitDepth)
 }
 
 /*
-  gdImageTiffCtx
-  --------------
-  Write the gd image as a tiff file
-  Parameters are:
-  image:  gd image structure;
-  out:    the stream where to write
- */
+	Function: gdImageTiffCtx
+
+	Write the gd image as a tiff file.
+
+	Parameters:
+
+		image - gd image structure;
+		out   - the stream where to write
+*/
 BGD_DECLARE(void) gdImageTiffCtx(gdImagePtr image, gdIOCtx *out)
 {
 	int clipx1P, clipy1P, clipx2P, clipy2P;
@@ -782,10 +784,11 @@ static int createFromTiffRgba(TIFF * tif, gdImagePtr im)
 	return GD_SUCCESS;
 }
 
-/* gdImageCreateFromTiffCtx
-** ------------------------
-** Create a gdImage from a TIFF file input from an gdIOCtx
- */
+/*
+	Function: gdImageCreateFromTiffCtx
+
+	Create a gdImage from a TIFF file input from an gdIOCtx.
+*/
 BGD_DECLARE(gdImagePtr) gdImageCreateFromTiffCtx(gdIOCtx *infile)
 {
 	TIFF *tif;
@@ -991,9 +994,9 @@ error:
 	return im;
 }
 
-/* gdImageCreateFromTIFF
-** ---------------------
- */
+/*
+	Function: gdImageCreateFromTIFF
+*/
 BGD_DECLARE(gdImagePtr) gdImageCreateFromTiff(FILE *inFile)
 {
 	gdImagePtr im;
@@ -1004,6 +1007,9 @@ BGD_DECLARE(gdImagePtr) gdImageCreateFromTiff(FILE *inFile)
 	return im;
 }
 
+/*
+	Function: gdImageCreateFromTiffPtr
+*/
 BGD_DECLARE(gdImagePtr) gdImageCreateFromTiffPtr(int size, void *data)
 {
 	gdImagePtr im;
@@ -1014,9 +1020,9 @@ BGD_DECLARE(gdImagePtr) gdImageCreateFromTiffPtr(int size, void *data)
 	return im;
 }
 
-/* gdImageTIFF
-** -----------
- */
+/*
+	Function: gdImageTiff
+*/
 BGD_DECLARE(void) gdImageTiff(gdImagePtr im, FILE *outFile)
 {
 	gdIOCtx *out = gdNewFileCtx(outFile);
@@ -1025,9 +1031,9 @@ BGD_DECLARE(void) gdImageTiff(gdImagePtr im, FILE *outFile)
 	out->gd_free(out);
 }
 
-/* gdImageTIFFPtr
-** --------------
- */
+/*
+	Function: gdImageTiffPtr
+*/
 BGD_DECLARE(void *) gdImageTiffPtr(gdImagePtr im, int *size)
 {
 	void *rv;
