@@ -51,12 +51,6 @@
  *--------------------------------------------------------------------------
  */
 
-/**
- * File: WBMP IO
- *
- * Read and write WBMP images.
- */
-
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
 #endif
@@ -206,17 +200,15 @@ BGD_DECLARE(gdImagePtr) gdImageCreateFromWBMPCtx(gdIOCtx *infile)
     A pointer to the new image or NULL if an error occurred.
 
   Example:
-    (start code)
 
-    gdImagePtr im;
-    FILE *in;
-    in = fopen("mywbmp.wbmp", "rb");
-    im = gdImageCreateFromWBMP(in);
-    fclose(in);
-    // ... Use the image ...
-    gdImageDestroy(im);
+    > gdImagePtr im;
+    > FILE *in;
+    > in = fopen("mywbmp.wbmp", "rb");
+    > im = gdImageCreateFromWBMP(in);
+    > fclose(in);
+    > // ... Use the image ...
+    > gdImageDestroy(im);
 
-    (end code)
 */
 
 BGD_DECLARE(gdImagePtr) gdImageCreateFromWBMP(FILE *inFile)
